/*
 * freetypestruct.h - internal header file.
 * List of functions that are exported from
 * the application to freetype
 *
 * Copyright (C) 2019 Thorsten Otto
 */

#ifndef __FREETYPESTRUCT_H__
#define __FREETYPESTRUCT_H__ 1

#ifndef __CDECL
#define __CDECL
#endif

#include <mint/slb.h>
#include <sys/types.h>
#include <unistd.h>
#include <setjmp.h>
#include <sys/stat.h>
#include <freetype/fttypes.h>

struct _freetype_funcs {
	/*
	 * sizeof(this struct), as
	 * used by the application.
	 */
	size_t struct_size;
	/*
	 * sizeof of an int of the caller.
	 * As for now, must match the one which
	 * was used to compile the library (32 bit)
	 */
	size_t int_size;
	/*
	 * version of freetype the caller used.
	 * As for now, should match the version that was
	 * used to compile the library.
	 */
	long freetype_vernum;
	
	const char *slbpath;
	SLB *zlib;
	SLB *bzip2;
	SLB *png;
	SLB *harfbuzz;
	short zlib_opened_here;
	short bzip2_opened_here;
	short png_opened_here;
	short harfbuzz_opened_here;
	long __CDECL (*p_slb_freetype_zlib_open)(void);
	long __CDECL (*p_slb_freetype_bzip2_open)(void);
	long __CDECL (*p_slb_freetype_png_open)(void);
	long __CDECL (*p_slb_freetype_harfbuzz_open)(void);

	void *__CDECL (*p_memset)(void *, FT_Int, size_t);
	void *__CDECL (*p_memcpy)(void *, const void *, size_t);
	FT_Int __CDECL (*p_memcmp)(const void *, const void *, size_t);
	void *__CDECL (*p_memchr)(const void *s, FT_Int c, size_t n);
	
	size_t __CDECL (*p_strlen)(const char *);
	char *__CDECL (*p_strcpy)(char *, const char *);
	char *__CDECL (*p_strncpy)(char *, const char *, size_t);
	char *__CDECL (*p_strcat)(char *, const char *);
	FT_Int __CDECL (*p_strcmp)(const char *, const char *);
	FT_Int __CDECL (*p_strncmp)(const char *, const char *, size_t);
	char *__CDECL (*p_strstr)(const char *, const char *);
	char *__CDECL (*p_strrchr)(const char *, FT_Int);
	long __CDECL (*p_strtol)(const char *, char **, FT_Int);

	void *__CDECL (*p_malloc)(size_t);
	void __CDECL (*p_free)(void *);
	void *__CDECL (*p_realloc)(void *, size_t);

	FILE *__CDECL (*p_fopen)(const char *pathname, const char *mode);
	FT_Int __CDECL (*p_fclose)(FILE *stream);
	size_t __CDECL (*p_fread)(void *ptr, size_t size, size_t nmemb, FILE *stream);
	size_t __CDECL (*p_fwrite)(const void *ptr, size_t size, size_t nmemb, FILE *stream);
	FT_Int __CDECL (*p_sprintf)(char *, const char *, ...);
	long __CDECL (*p_ftell)(FILE *stream);
	FT_Int __CDECL (*p_fseek)(FILE *stream, long offset, FT_Int whence);
	
	FT_Int __CDECL (*p_sigsetjmp)(jmp_buf, FT_Int);
	__attribute__((__noreturn__)) void __CDECL (*p_longjmp)(jmp_buf, FT_Int);

	void __CDECL (*p_qsort)(void *base, size_t nmemb, size_t size, FT_Int __CDECL (*compar)(const void *, const void *));
	char *__CDECL (*p_getenv)(const char *name);

	/* room for later extensions */
	void *unused[24];
};

extern struct _freetype_funcs freetype_funcs;

long __CDECL freetype_slb_control(long fn, void *arg);
struct _freetype_funcs *freetype_get_slb_funcs(void);

#endif
